package Maui::Client;

#
# Copyright (c) 2002 Alex Martin <A.J.Martin@qmul.ac.uk>
#
# All rights reserved.  This program is free software;
# you can redistribute it and/or modify it under the same terms
# as Perl itself.

use 5.006;
use strict;
use warnings;
use Carp;
#use Time::Local;

require IO::Socket;
require Maui;

our $VERSION = '0.05';

our @ISA = qw(Exporter DynaLoader);

#enum {
#  oNONE = 0,
#  oJob,
#  oNode,
#  oRes,
#  oUser,
#  oGroup,
#  oAccount,
#  oPartition,
#  oScheduler,
#  oSystem,
#  oClass,
#  oQOS };
our $oTypes = { None => 0, Job => 1, Node => 2, Reservation => 3, User => 4, Group => 5,
                Account => 6, Partition => 7, Scheduler => 8, System => 9, Class => 10,
                QOS => 11 };
#
# enum { ptDEFAULT = 0, ptOFF, ptON, ptSOFT, ptHARD, ptEXEMPT, ptQUEUE };
#

our $ptTypes = { Default => 0, Off => 1, On => 2, Soft => 3, Hard => 4, Exempt => 5 ,
                Queue => 6 };

#enum {
#  modeNONE = 0,
#  modeRelative,
#  modeSummary,
#  modeGrep,
#  modeVerbose,
#  modeForce,
#  modeParse,
#  modeForce2,
#  modeComplete,
#  modeBlock,
#  modeNonBlock,
#  modeOverlap,
#  modeClear };

our $modes = { None => 0, Relative => 1, Summary => 2, Grep => 3, Verbose => 4, Force => 5,
                Parse => 6, Force2 => 7, Complete => 8, Block => 9, NonBlock => 10,
                Overlap => 11, Clear => 12 };

#enum {
#  diagFairShare = 0,
#  diagJob,
#  diagNode,
#  diagFrame,
#  diagPartition,
#  diagPriority,
#  diagQueue,
#  diagQOS,
#  diagReservation,
#  diagUser,
#  diagGroup,
#  diagAccount,
#  diagClass
#  };

our $diagmodes = { FairShare => 0, Job => 1, Node => 2, Frame => 3, Partition => 4, Priority => 5,
                Queue => 6, QOS => 7, Reservation => 8, User => 9, Group => 10,
                Account => 11, Class => 12 };

# enum {
#  cmpNONE = 0,
#  cmpLT,
#  cmpLE,
#  cmpEQ,
#  cmpGE,
#  cmpGT,
#  cmpNE,
#  cmpSUB
#  };

our $cmpops = { NONE => 0, "<" => 1, "<=" => 2, "==" => 3, ">="  => 4, ">" => 5, "!=" => 6, "=~" => 7 };

#enum {
#  mjpNONE = 0,
#  mjpRelative,
#  mjpAbsolute };

our $prioritymodes = { NONE => 0, Relative => 1, Absolute => 2 };

#enum { hUser = 0,hSystem,hBatch,hAll };

our $holdtypes = { User => 0, System => 1, Batch => 2, All => 3 };

#enum MetaCtlCmdEnum {
#  mcNONE = 0,
#  mcInitialize,
#  mcCommit,
#  mcList,
#  mcSet,
#  mcRemove,
#  mcQuery,
#  mcSubmit,
#  mcModify,
#  mcResetStats,
#  mcRegister };

our $metactlcmds = { NONE => 0, Initialize => 1, Commit => 2, List => 3, Set => 4, Remove => 5,
                    Query => 6, Submit => 7, Modify => 8, ResetStats => 9, Register => 10 };


#enum JobCtlCmdEnum {
#  mjcmNONE = 0,
#  mjcmCancel,
#  mjcmModify,
#  mjcmQuery,
#  mjcmResume,
#  mjcmSuspend };

our $jobctlcmds = { NONE => 0, Cancel => 1, Modify => 2, Query => 3, Resume => 4, Suspend => 5 };

#enum {
#  schedKill = 0,
#  schedResume,
#  schedReconfig,
#  schedStep,
#  schedStop,
#  schedNodeTable,
#  schedSubmit,
#  schedList,
#  schedModify };

our $schedmodes = { Kill => 0, Resume => 1, Reconfig => 2, Step => 3, Stop => 4, NodeTable => 5,
                    Submit => 6, List => 7, Modify => 8 };

sub new {
    my $class = shift;
    my $this = {
        PeerAddr      => "localhost",
        PeerPort      => 42559,
        Proto         => "tcp",
        Reuse         => 1,
        Mode          => "raw",
        Auth          => getpwuid($>),
        @_
    };
    bless $this,$class;
    if( $this->{Cksum_Seed} ) { $Maui::Msg::CKSUM_SEED = $this->{Cksum_Seed} };
    if( $this->{ClientName} ) { $Maui::Msg::CLIENT     = $this->{ClientName} };


#    $this->{Sock} = IO::Socket::INET -> new(PeerAddr => $this->{PeerAddr},
#        PeerPort => $this->{PeerPort},
#        Proto    => $this->{Proto} ) || return 0;

    return $this;
}



sub do_command {
    my $this = shift;
    my ( $cmd, $arg ) = @_;
    print "cmd $cmd with args $arg \n";
    my $sock = IO::Socket::INET -> new(PeerAddr => $this->{PeerAddr},
        PeerPort => $this->{PeerPort},
        Proto    => $this->{Proto} ) || return 0;
    my $msg = new Maui::Msg( CMD =>"$cmd" , ARG => "$arg", AUTH => $this->{Auth} );
    $msg->send( $sock ); # != 0 )return -1;
    my $reply = Maui::Msg::receive( $sock );
    return $reply;
}

sub canceljob {
    my $this = shift;
    my $args = {
        JobIDs => "",
        @_
    };
   my $reply = $this->do_command( "canceljob", $args->{JobIDs} );
   return $reply->{SC}, $reply->{ARG};
}

sub changeparam {
    my $this = shift;
    my $args = {
        Parameter => "",
        Value => "",
        @_
    };
    my $reply = $this->do_command( "changeparam", $args->{Parameter}." ".$args->{Value}."\n" );
    return $reply->{SC}, $reply->{ARG};
}

sub checkjob {
    my $this = shift;
        my $args = {
        PolicyLevel => "Off",
        JobID  => 0,
        ReservationList => "",
        Verbose => 0,
        @_
    };
    my $flags = 0;
    $flags |=  $args->{Verbose} ?  1<<$modes->{Verbose} : 0;

    my $reply = $this->do_command( "checkjob", $ptTypes->{$args->{PolicyLevel}}." ".$args->{JobID}." "
        .$flags." ".$args->{ReservationList} );
    return $reply->{SC}, $reply->{ARG};
}


sub checknode {
    my $this = shift;
    my $args = {
        NodeID => 0,
        @_
    };
    my $reply = $this->do_command( "checknode", $args->{NodeID} );
    return $reply->{SC}, $reply->{ARG};
}

sub diagnose {
    my $this = shift;
    my $args = {
        ObjectType  => "",
        ObjectID    => "[NONE]",
        PolicyLevel => "Off",
        Partition => "All",
        Verbose => 0,
        @_
    };
    my $flags = 0;
    $flags |=  $args->{Verbose} ?  1<<$modes->{Verbose} : 0;

    my $arg="";
    if ( $args -> {ObjectType } eq "Queue" ){
        $arg = $diagmodes -> {$args -> {ObjectType}}." ".$ptTypes -> {$args->{PolicyLevel}}." ".
            $args->{Partition};
    } else {
        $arg = $diagmodes -> {$args -> {ObjectType}}." ".$flags." ".$args->{Partition}." ".
            $args->{ObjectID};
    }
    my $reply = $this->do_command( "diagnose",$arg );
    return $reply->{SC}, $reply->{ARG};
}

sub jobctl {
    my $this = shift;
    my $args = {
        JobID => "",
        Mode  => "Cancel",  # or Suspend
        @_
    };
    my $reply = $this->do_command( "jobctl", $args->{JobID}." ".$jobctlcmds -> {$args->{Mode}} );
    return $reply->{SC}, $reply->{ARG};
}

sub metactl {
    my $this = shift;
    my $args = {
        Mode  => "NONE", # Commit, List, Modify, Remove, Submit, Set, Query
        Args  => "",
        @_
    };
    my $reply = $this->do_command( "metactrl", $metactlcmds -> {$args->{Mode}}." ".
        $args->{Args} );
    return $reply->{SC}, $reply->{ARG};

}

sub migratejob {
    my $this = shift;
    my $args = {
        JobID   => "",
        Server  => "",
        @_
    };
    my $reply = $this->do_command( "migratejob", $args->{JobID}." ".$args->{Server} );
    return $reply->{SC}, $reply->{ARG};
}

sub nodectl {
    my $this = shift;
    my $args = {
        Mode  => "", # create, destroy, modify
        Args  => "",
        NodeIDs => "",
        @_
    };
    my $reply = $this->do_command( "nodectrl", $args->{Mode}." ".$args->{Args}." ".
        $args->{NodeIDs} );
    return $reply->{SC}, $reply->{ARG};
}


sub query {
    my $this = shift;
    my $args = {
        Args  => "",
        @_
    };
    my $reply = $this->do_command( "query", $args->{Args} );
    return $reply->{SC}, $reply->{ARG};
}

sub releasedeadline {
    my $this = shift;
    my $args = {
        JobID => "",
        @_
    };
    my $reply = $this->do_command( "releasedeadline", $args->{JobID} );
    return $reply->{SC}, $reply->{ARG};
}

sub releasehold {
    my $this = shift;
    my $args = {
        JobID     => "",
        HoldType  => "ALL",
        @_
    };
    my $reply = $this->do_command( "releasehold",$args->{JobID}." ".$holdtypes->{$args->{HoldType}} );
    return $reply->{SC}, $reply->{ARG};
}

sub releaseres {
    my $this = shift;
    my $args = {
        ResID => "",
        @_
    };
    my $reply = $this->do_command( "release",$args->{ResID} );
    return $reply->{SC}, $reply->{ARG};
}

sub resetstats {
    my $this = shift;
    my $reply = $this->do_command( "resetstats", "" );
    return $reply->{SC}, $reply->{ARG};
}

sub runjob {
    my $this = shift;
    my $args = {
        JobID     => "",
        NodeList  => "[NONE]",
        Partition => "All",
        Mode => "NONE",   # "CLEAR" || "BLOCK" || "FORCE" || "FORCE2"
    };
    my $reply = $this->do_command( "runjob", $args->{JobID}." ".$args->{Mode}." ".
         $args->{Partition}." ",$args->{Mode} );
    return $reply->{SC}, $reply->{ARG};
}

sub schedctl {
    my $this = shift;
    my $args = {
        Mode  => "",  # Submit, Kill, List, Modify, NodeTable, Resume, Reconfig, Stop, Step
        Arg   => "",
        @_
    };
    my $reply = $this->do_command( "schedctl", $schedmodes -> {$args->{Mode}}." ".$args->{Arg} );
    return $reply->{SC}, $reply->{ARG};
}

sub setdeadline {
    my $this = shift;
    my $args = {
        JobID => "",
        Deadline => 0,
        @_
    };
    my $reply = $this->do_command( "setdeadline", $args->{JobID}," ".$args->{Deadline} );
    return $reply->{SC}, $reply->{ARG};
}

sub sethold {
    my $this = shift;
    my $args = {
        JobID     => "",
        HoldType  => "ALL",
        @_
    };
    my $reply = $this->do_command( "sethold",$args->{JobID}." ".$holdtypes->{$args->{HoldType}} );
    return $reply->{SC}, $reply->{ARG};
}

sub setqos {
    my $this = shift;
    my $args = {
        JobID    => "",
        QOS      => 0,
        @_
    };
    my $reply = $this->do_command( "setqos",$args->{JobID}." ".$args->{QOS} );
    return $reply->{SC}, $reply->{ARG};
}

sub setspri {
    my $this = shift;
    my $args = {
        JobID    => "",
        Priority => 0,
        Mode     => "Relative",
        @_
    };
    my $reply = $this->do_command( "setspri",$args->{JobID}." ".$args->{Priority}." ".
        $prioritymodes -> { $args->{Mode} } );
    return $reply->{SC}, $reply->{ARG};
}

sub setupri {
    my $this = shift;
    my $args = {
        JobID    => "",
        Priority => 0,
        Mode     => "Relative",
        @_
    };
    my $reply = $this->do_command( "setupri",$args->{JobID}." ".$args->{Priority}." ".
        $prioritymodes -> { $args->{Mode} } );
    return $reply->{SC}, $reply->{ARG};
}

sub setres {
    my $this = shift;
    my $now = time();
    my $args = {
        ClientTime     => $now,
        StartTime      => 0,
        Duration       => 3600,
        Partition      => "All",
        UserList       => "[NONE]",
        GroupList      => "[NONE]",
        AccountList    => "[NONE]",
        ClassList      => "[NONE]",
        QOSList        => "[NONE]",
        Name           => "[NONE]",
        ResourceDes    => "Procs=-1",
        ChargeSpec     => "[NONE]",
        ResourceExp    => "[NONE]",
        FeatureList    => "[NONE]",
        NodeSetList    => "",
        Flags          => "[NONE]",
        @_
    };
    my $begintime;
    if ( $args->{StartTime} =~ s/^\+// ){
        $begintime = $now + $args->{StartTime};
    } else {
        $begintime = $args->{StartTime};
    }
    my $endtime = $begintime + $args->{Duration};

    my $reply = $this->do_command( "setres", $args->{ClientTime}." ".$begintime." ".
        $args->{Partition}." ".$args->{UserList}." ".$args->{GroupList}." ".
        $args->{AccountList}." ".$args->{ClassList}." ".$args->{QOSList}." ".
        $args->{Name}." ".$args->{ResourceDes}." ".$args->{ChargeSpec}." ".$args->{ResourceExp}." ".
        $args->{FeatureList}." ".$args->{NodeSetList}." ".$args->{Flags} );
    return $reply->{SC}, $reply->{ARG};

}

sub showbf {
    my $this = shift;
    my $args = {
        UserName    => "[ALL]",
        Group       => "[ALL]",
        Account     => "[ALL]",
        Partition   => "All",
        Duration    => 0,
        NodeCount   => 0,
        ProcCount   => 0,
        DMemory     => 0,
        Memory      => ">= 0",
        BFMode      => 0,
        Class       => "[NONE]",
        FeatureList => "[NONE]",
        QOS         => "[NONE]",
        Verbose     => 0,
        @_
    };
    my $flags = 0;
    $flags |=  $args->{Verbose} ?  1<<$modes->{Verbose} : 0;

    my @mem = split (/\s+/,$args->{Memory});
    if ( $#mem == 0 ) {
        $mem[1]=$mem[0];
        $mem[0]=">=";
    }


    my $reply = $this->do_command( "showbf", $args->{UserName}." ".$args->{Group}." ".
        $args->{Account}." ".$args->{Partition}." ".$args->{Duration}." ".$args->{NodeCount}." ".
        $args->{ProcCount}." ".$args->{DMemory}." ".$mem[1]." ".$cmpops->{$mem[0]}." ".
        $args->{BFMode}." ".$flags." ".$args->{Class}." ".$args->{FeatureList}." ".
        $args->{QOS}." " );
    return $reply->{SC}, $reply->{ARG};
}

sub showconfig {
    my $this = shift;
    my $args = {
        Verbose => 0,
        @_
    };
    my $verb = $args->{Verbose} ? "VERBOSE" : "";
    my $reply = $this->do_command( "showconfig", $verb );
    return $reply->{SC}, $reply->{ARG};
}

sub showdeadline {
    my $this = shift;
    my $args = {
        JobID => "",
        @_
    };
    my $reply = $this->do_command( "showdeadline", $args->{JobID} );
    return $reply->{SC}, $reply->{ARG};
}

sub showgrid {
    my $this = shift;
    my $args = {
        StatisticType => "",
        @_
    };
    my $reply = $this->do_command( "showgrid", uc($args->{StatisticType}) );
    return $reply->{SC}, $reply->{ARG};
}

sub showhold {
    my $this = shift;
    my $reply = $this->do_command( "showhold", "" );
    return $reply->{SC}, $reply->{ARG};
}

sub showq {
    my $this = shift;
    my $args = {
        QueueMode => 0,
        Partition => "ALL",
        @_
    };
    my $reply = $this->do_command( "showq", $args->{QueueMode}." ".$args->{Partition} );

    if( $this->{Mode} =~ "raw" ){
          return $reply->{SC}, $reply->{ARG};
    }

    my @lines = split( /\n/, $reply->{ARG} );
    my $jobstat    = "Active";
    my $response = "<response ";
    for ( my $i=0;  $i < $#lines; $i++){

        $lines[$i] =~ s/^\s+//;
        my @tmp  = split( /\s+/, $lines[$i]);

        if ( $i == 0 ){
                $response .= "presenttime=\"$tmp[0]\" ";
                $response .= "upprocs=\"$tmp[1]\" ";
                $response .= "idleprocs=\"$tmp[2]\" ";
                $response .= "upnodes=\"$tmp[3]\" ";
                $response .= "idlenodes=\"$tmp[4]\" ";
                $response .= "activenodes=\"$tmp[5]\" ";
                $response .= "busyprocs=\"$tmp[6]\" >\n";
        } elsif ( $tmp[0] =~ /ENDACTIVE/ ) {
                $jobstat = "Idle";
        } elsif ( $tmp[0] =~ /ENDIDLE/ ) {
                $jobstat = "NotQueued";
        } else {
                $response .= "\t<$jobstat"."job ";
                $response .= "name=\"$tmp[0]\" ";
                $response .= "username=\"$tmp[1]\" ";
                $response .= "starttime=\"$tmp[2]\" ";
                $response .= "queuetime=\"$tmp[3]\" ";
                $response .= "procs=\"$tmp[4]\" ";
                $response .= "cpulimit=\"$tmp[5]\" ";
                $response .= "tmpqos=\"$tmp[6]\" ";
                $response .= "state=\"$tmp[7]\" ";
                $response .= "priority=\"$tmp[8]\" />\n";
        }
    }
    $response .= "</response>\n";
    return $reply->{SC}, $response;
}

sub showres {
    my $this = shift;
    my $args = {
        ObjectType => "Job",
        ResID      => "[NONE]",
        Partition  => "All",
        Grep       => 0,
        Relative   => 0,
        Summary    => 0,
        Verbose    => 0,
        @_
    };

    my $flags = 0;
    $flags |=  $args->{Grep} ?  1<<$modes->{Grep} : 0;
    $flags |=  $args->{Relative} ?  1<<$modes->{Relative} : 0;
    $flags |=  $args->{Summary} ?  1<<$modes->{Summary} : 0;
    $flags |=  $args->{Verbose} ?  1<<$modes->{Verbose} : 0;
    my $reply = $this->do_command( "showres",$oTypes->{$args->{ObjectType}}." ".$args->{Partition}." ".
        $flags." ".$args->{ResID} );
    return $reply->{SC}, $reply->{ARG};

}

sub showstart {
    my $this = shift;
    my $args = {
        JobID => "",
        @_
    };
    my $reply = $this->do_command( "showstart", $args->{JobID} );
    return $reply->{SC}, $reply->{ARG};
}

sub showstate {
    my $this = shift;
    my $reply = $this->do_command( "showstate", "" );
    return $reply->{SC}, $reply->{ARG};
}


sub showstats {
    my $this = shift;
    my $args = {
        ObjectType => "Scheduler",  # Account || Group || Node || Scheduler || User
        ObjectID   => 0,
        Partition  => "ALL",
        Verbose    => 0,
        @_
    };

    my $flags = 0;
    $flags |=  $args->{Verbose} ?  1<<$modes->{Verbose} : 0;

    my $reply = $this->do_command( "showstats",$oTypes->{$args->{ObjectType}}." ".$args->{ObjectID}." "
        .$args->{Partition}." ".$flags );
    return $reply->{SC}, $reply->{ARG};
}


1;
__END__
