package Maui::Wiki;

#
# Copyright (c) 2002 Alex Martin <A.J.Martin@qmul.ac.uk>
#
# All rights reserved.  This program is free software;
# you can redistribute it and/or modify it under the same terms
# as Perl itself.

use 5.006;
use strict;
use warnings;
use Carp;
use IO::Socket;
use Socket;
require Maui;

our $VERSION = '0.05';

our @ISA = qw(Exporter DynaLoader);
sub new {
    my $class = shift;
    my $this = {
        LocalPort     => 44321,
        Type          => SOCK_STREAM,
        Reuse         => 1,
        Listen        => 10,
        Proto         => "tcp",
        Debug         => 0,
        GetNodes      => \&not_supported,
        GetJobs       => \&not_supported,
        StartJob      => \&not_supported,
        CancelJob     => \&not_supported,
        @_
    };
    bless $this,$class;
    if( $this->{Cksum_Seed} ) { $Maui::Msg::CKSUM_SEED = $this->{Cksum_Seed} };
    if( $this->{ClientName} ) { $Maui::Msg::CLIENT     = $this->{ClientName} };
    bless $this,$class;


    $this->{Sock} = IO::Socket::INET -> new( LocalPort => $this->{LocalPort},
                                                 Type      => $this->{Type},
                                                 Reuse     => $this->{Reuse},
                                                 Listen    => $this->{Listen} ) || return 0;

	return $this;
}

sub process  {
    my $this = shift;
    my $client;
    while ( $client = $this -> {Sock} ->accept() ){
        my $thismsg =  Maui::Msg::receive($client);

        if( $this->{Debug} ){
            print "< CMD: ".$thismsg->{CMD}," ARG: ".$thismsg->{ARG},"\n";
        }
        my $reply;
        if( $thismsg -> {'CMD'} eq 'GETNODES' ){
            $reply =  $this -> {GetNodes} -> ( split ( /:/, $thismsg->{ARG} ) );
        } elsif ( $thismsg -> {'CMD'} eq 'GETJOBS' ){
            $reply =  $this -> {GetJobs} -> ( split ( /:/, $thismsg->{ARG} ) );
        } elsif ( $thismsg -> {'CMD'} eq 'STARTJOB' ){
            $reply = $this -> {StartJob} -> (split( /\s+TASKLIST=/,$thismsg->{ARG},2));
        } elsif ( $thismsg -> {'CMD'} eq 'CANCELJOB' ){
            $reply = $this -> {CancelJob} -> (split( /\s+TYPE=/,$thismsg->{ARG},2));
        } else {
            $reply = not_supported();
        }
        $reply->send($client);

#		if( $this->{Debug} ){
            print "> REPLY: ",$reply -> {DT},"\n\n";
#		}

    }
    return -1;
}

sub not_supported {
    return new Maui::Msg ( SC => -1, ARG => "Function not supported" );
}

1;
__END__
# Below is stub documentation for your module. You better edit it!

=head1 NAME

Maui::Wiki - Perl interface to Maui scehduler

=head1 SYNOPSIS

  use Maui::Wiki;

=head1 DESCRIPTION

Stub documentation for Checksum, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.


=head1 AUTHOR

Alex Martin<lt>a.j.martin@qmul.ac.uk<gt>

=head1 SEE ALSO

L<perl>.

=cut
